// Programmer: Maura Dailey
// CSE 120
// Section #: 1
// Date: 04/08/02
//
// Brief: Functions MaxValue() and MaxCalc() are utilized to find
//   the sum of the path with the greatest value in an unsorted, but
//   complete, binary tree. This tree must have a row of zeroes to
//   distinguish its bottom row from its other rows, and is stored
//   in an array of integers. In this program, the function that calls
//   these functions is main.
//
// MaxValue() is called with an array of ints initialized according to
// the specification mentioned previously. It initializes several
// necessary variables and passes them to another function MaxCalc()
// which calculates the actual max path. It returns the results of this
// function to that which calls it
//
// MaxCalc() is called by MaxValue() with its variables initialized
// to their proper states. Using recursion, it travels down every path,
// keeping tabs on the total sum and previous greatest sum at each
// point. When it reaches the end, the greatest path for the binary tree
// is returned to MaxValue() which returns it in this case to main.

#include <iostream>

int MaxValue(int tree[]);
// Pre : Array tree has been defined and values assigned, bottom row
//       set to all zeroes.
// Post: MaxCalc() has been called and the maximum value determined
//       by MaxCalc() is returned to MaxValue()

int MaxCalc(int tree[], int pos, int row, int sum, int& current_max);
// Pre : Variables position, row, sum and current_max are initialized
//       to their proper values. Tree passed in must be a complete tree
//       with bottom row consisting of all zeroes.
// Post: Maximum path value has been returned to MaxValue()

int main()
{
  int tree1[10] = {4,2,5,1,8,3,0,0,0,0};
  int tree2[21] = {5,3,1,9,7,8,5,7,9,5,4,6,8,2,4,0,0,0,0,0,0};
  int tree3[21] = {7,4,6,3,5,1,4,8,2,4,3,4,9,2,5,0,0,0,0,0,0};
  int tree4[21] = {5,3,1,9,7,100,5,7,9,5,4,6,8,2,4,0,0,0,0,0,0};

  int max;

  max = MaxValue(tree1);
  cout << "The max value for tree1 is " << max << endl;
  
  max = MaxValue(tree2);
  cout << "The max value for tree2 is " << max << endl;

  max = MaxValue(tree3);
  cout << "The max value for tree3 is " << max << endl;

  max = MaxValue(tree4);
  cout << "The max value for tree4 is " << max << endl;
  
  return 0;
}

int MaxValue(int tree[])
{
  // Initialize the variables that follow. Variables position and row
  // will track the current position of the node and decide how to
  // access the next node in the tree. Sum will increment at each
  // new node, and current_max will be the maximum path at the end
  // of the tree.

  // position will be the actual position inside the array, row will
  // be a helper variable that will help determine how pos should
  // increment to go left or right on a tree.

  int position = 0;
  int row = 1;
  int sum = 0;
  int current_max = 0;

  // max_path will be used to store the maximum path value returned
  // by MaxCalc()

  int max_path;

  // MaxCalc() is a value returning recursive function that takes the
  // initialized variables and returns the maximum path. The value
  // returned will be stored in max_path.

  max_path = MaxCalc(tree, position, row, sum, current_max);
  
  // Return max_path
  return max_path;
}

int MaxCalc(int tree[], int position, int row, int sum, int& current_max)
{
  // Base Case: If the value of the current node at pos is zero, then
  //   you must be at the bottom of a path. Check to see if that path's
  //   sum is greater than the last greatest sum. Return greatest sum.

  if (tree[position]==0)
  {
    if (sum >= current_max)
    {
      current_max = sum;
      return current_max;
    }
    else
      return current_max;
  }

  // General Case: First travels down the left path, then the right
  //   path. Increments max, and passes in the incremented max along
  //   with a modified position and row to move the position to the proper
  //   location, be it left or right.

  else
  {
    sum += tree[position];
    MaxCalc(tree, (position+row), (row+1), sum, current_max);
    MaxCalc(tree, (position+row+1), (row+1), sum, current_max);
  }
}